#include "Arduino.h"
#include "ESP8266WiFi.h"
#include "ESP8266WiFiMulti.h"
#include "SPI.h"
#include "ESP8266HTTPClient.h"
#include "MQTT.h"
#include "config.h"

//Funktionsdeklarationen
String readRequest(WiFiClient* client);
char key;
char MQTTLWT = false;
char MQTTKEY;
char KEY_CK;

//Initalisieren
WiFiServer server = WiFiServer(HSRPRT);
WiFiClient client;

// Create the Keypad
#include <Keypad.h>
Keypad kpd = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

ESP8266WiFiMulti WiFiMulti;
HTTPClient http;

//===============================MQTT STUFF=============================
MQTT myMqtt(host, EXMQ, EXMPRT);
void myDataCb(String& topic, String& data);
void myPublishedCb();
void myDisconnectedCb();
void myConnectedCb();

void myConnectedCb()
{
  Serial.println("MQTT Server connected!");
  Serial.println("IP: "+(String)EXMQ+" Port: "+(String)EXMPRT);
  MQTTLWT = true;   
}

void myDisconnectedCb()
{
  MQTTLWT = false;
  Serial.println("Server "+(String)EXMQ+":"+(String)EXMPRT+" disconnected, try to reconnect...");
  delay(500);
  myMqtt.connect();
}

void myPublishedCb()
{
  //Serial.println("published.");
}

void myDataCb(String& topic, String& data)
{
    //Serial.println("MQTT received:"+topic+" "+data);
	MQTTKEY = data.charAt(0);
	Serial.println("MQTT received:"+(String)MQTTKEY);
}

//===============================SETUP==================================
void setup()
{
  Serial.begin(115200);
  Serial.println();
  Serial.println();
  
  // We start by connecting to a WiFi network
  Serial.print("Connecting to ");
  Serial.println(ssid);

  if (STATIC) {
	WiFi.config(ip, gw, gw, su);
  }
  WiFi.softAPdisconnect(true);
  WiFi.hostname(host);
  WiFi.begin(ssid, pass);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected!");
  Serial.println("IP: " + WiFi.localIP().toString());
  Serial.println();
  
  // allow reuse (if server supports it)
  http.setReuse(true);
  server.begin(); 
  Serial.println("HTTP Server started");
  Serial.println();

  // Start the MQTT Client   
  if (EXSERV) { 
  // setup callbacks
  myMqtt.onConnected(myConnectedCb);
  myMqtt.onDisconnected(myDisconnectedCb);
  myMqtt.onPublished(myPublishedCb);
  myMqtt.onData(myDataCb);
  myMqtt.connect();
  delay(500);
  myMqtt.subscribe("cmnd/"+(String)host+"/KEY");
  Serial.println("subscribe: cmnd/"+(String)host+"/KEY");
  Serial.println();
  }
}

void loop()
{
  Key_Input();
  Switchserver();
}

void Key_Input() {
  key = kpd.getKey();
  if (key) {
    KEY_CK = key;
    KeyCheck();
  } 
  if (MQTTKEY != false) {
	KEY_CK = MQTTKEY;
	KeyCheck();
	MQTTKEY = false;
  } 
}

void KeyCheck()
{
   String valueStr(KEY_CK);
   
   switch (KEY_CK) {
   //=============================== Matrix 3x4 ====================================
   case '1':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
       if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_1);
         int httpCode = http.GET();
         http.end();
       }
       break;
       
   case '2':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
       if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_2);
         int httpCode = http.GET();
         http.end();
       }
       break;

    case '3':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
       if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_3);
         int httpCode = http.GET();
         http.end();
       }
       break;
       
    case '4':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
       if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_4);
         int httpCode = http.GET();
         http.end();
       }
       break;
       
    case '5':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
       if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_5);
         int httpCode = http.GET();
         http.end();
       } 
       break;
       
    case '6':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
       if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_6);
         int httpCode = http.GET();
         http.end();
       }
       break;
       
    case '7':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
      if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_7);
         int httpCode = http.GET();
         http.end();
      }
      break;
      
    case '8':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
      if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_8);
         int httpCode = http.GET();
         http.end();
      }
      break;   
     
    case '9':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
      if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_9);
         int httpCode = http.GET();
         http.end();
      }
      break;
      
    case '0':
      Serial.println(KEY_CK);
      if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_0);
         int httpCode = http.GET();
         http.end();
      }
      break;
      
    case '#':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
      if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_A);
         int httpCode = http.GET();
         http.end();
       } 
       break;
       
    case '*':
       Serial.println(KEY_CK);
       if(MQTTLWT) { 
		 myMqtt.publish("stat/"+(String)host+"/KEY",valueStr);		 
       } 
      if((WiFiMulti.run() == WL_CONNECTED)) {
         http.begin(KEY_B);
         int httpCode = http.GET();
         http.end();
      }
      break;
      
    default:
      Serial.println(KEY_CK);
      break;
    }    
}

void Switchserver() {
  
  size_t size;
  if (WiFiClient client = server.available()) {
    
  
    //Anfrage empfangen
    String request = readRequest(&client);
                
                //Debug Ausgabe
                if(SERIAL_DEBUG) {
                                  
            Serial.println("request " + request);
          }
                
                // Variablen Aufteilen
                // Data is now available in pieces array
                // pieces[0] is first item
                // pieces[1] is second item, and so on
                // You can call toInt() on the data to convert it to an int
                // ex. int value = pieces[0].toInt();
          const int numberOfPieces = 6;
          String pieces[numberOfPieces];
          int counter = 0;
          int lastIndex = 0;
   
          for (int i = 0; i < request.length(); i++) {
    
            if (request.substring(i, i + 1) == ":") {
              pieces[counter] = request.substring(lastIndex, i);
              lastIndex = i + 1;
              counter++;
            }
          }
                pieces[counter] = request.substring(lastIndex, request.length() + 1);

                //Anfrage bearbeiten
                int type = pieces[0].toInt();
                if(type == 2) {
                  
                        //GPIO Ausgang schalten Typ 2
                        //(Typ[0]:Pin[1]:Befehl[2]) + Leerzeichen am Ende
                        int pin = pieces[1].toInt();
                        int command = pieces[2].toInt();
                        
                        pinMode(pin, OUTPUT);
                        if(command == 1) {
                              
                                digitalWrite(pin, HIGH);
                        } else {
                                
                                digitalWrite(pin, LOW);
                        }
                        
                        //Debug Ausgabe
                        if(SERIAL_DEBUG) {
                                  
                    Serial.print("write ");
                                Serial.print(pin);
                                Serial.print(" ");
                                Serial.println(command);
                  }
                } else if(type == 3) {
                  
                        //GPIO Eingang schalten Typ 3
                        //(Typ[0]:Pin[1]) + Leerzeichen am Ende
                        int pin = pieces[1].toInt();
                        
                        pinMode(pin, INPUT);
                        int state = digitalRead(pin);
                        client.println(state);
                        client.flush();
                        
                        //Debug Ausgabe
                        if(SERIAL_DEBUG) {
                                  
                    Serial.print("read ");
                                Serial.print(pin);
                                Serial.print(" state ");
                                Serial.println(state);
                  }
                }
    
                //verbindung beenden
    client.stop();

  }

}

String readRequest(WiFiClient* client) {
  
  String request = "";
  // Loop while the client is connected.
  while (client->connected()) {
    // Read available bytes.
    while (client->available()) {
      // Read a byte.
      char c = client->read();

      // Exit loop if end of line.
      if ('\n' == c) {
        return request;
      }
      // Add byte to request line.
      request += c;
    }
  }
  return request;
}

